import contextlib
from operator import itemgetter

import click
from flask import Config, current_app
from flask.cli import cli, with_appcontext

from .utils.click import echo_table, secho
from .utils.helpers import DEFAULT_SENSITIVE_CONFIG_KEYS, filter_dict, get_blueprint_rel_path, mask_sensitive_values


@cli.command("blueprints", short_help="Show blueprints informations.")
@click.option("--sort", "-s", is_flag=True, help="Show blueprints info sorted by name.")
@with_appcontext
def blueprints_cmd(sort: bool) -> None:
    """Display information about registered Flask blueprints.

    Args:
        sort (bool): Sort blueprints by name.
    """
    blueprints = list(current_app.iter_blueprints())
    if not blueprints:
        secho("No blueprints were registered.", category="error")
        raise click.Abort()

    rows = [
        (
            blueprint.name,
            blueprint.import_name,
            str(get_blueprint_rel_path(blueprint)),
            str(blueprint.url_prefix or ""),
            str(blueprint.subdomain or ""),
        )
        for blueprint in blueprints
    ]
    headers = ["Name", "Import Name", "Relative Path", "URL Prefix"]
    if any(blueprint.subdomain for blueprint in blueprints):
        headers.append("Subdomain")

    rows = sorted(rows) if sort else rows
    echo_table(
        rows,  # type: ignore
        tuple(headers),
        current_app.config.get("FLASK_CLI_EXTEND_TABLE_COLORS"),
    )


@cli.command("routes", short_help="Show the routes for the app.")
@click.option(
    "--sort",
    "-s",
    type=click.Choice(("endpoint", "methods", "domain", "rule", "match")),
    default="endpoint",
    help=("Method to sort routes by. 'match' is the order that Flask will match routes when dispatching a request."),
)
@click.option("--blueprint", "-b", type=click.STRING, help="Show the routes for the blueprint.")
@click.option("--all-methods", "-A", is_flag=True, help="Show HEAD and OPTIONS methods.")
@with_appcontext
def routes_cmd(sort: str, all_methods: bool, blueprint: str) -> None:
    """Show all registered routes with endpoints and methods.

    Args:
        sort (str): Method to sort routes by.
        all_methods (bool): Show HEAD and OPTIONS methods.
        blueprint (str): Show routes for a specific blueprint.
    """
    url_map = current_app.url_map
    rules = list(url_map.iter_rules())
    if not rules:
        secho("No routes were registered.", category="error")
        raise click.Abort()

    ignored_methods = set() if all_methods else {"HEAD", "OPTIONS"}
    host_matching = url_map.host_matching
    has_domain = any(rule.host if host_matching else rule.subdomain for rule in rules)
    rows = [
        (
            rule.endpoint,
            ", ".join(sorted((rule.methods or set()) - ignored_methods)),
            rule.rule,
            str(rule.host if host_matching else rule.subdomain or ""),
        )
        for rule in rules
        if not blueprint or rule.endpoint.startswith(blueprint)
    ]

    headers = ["Endpoint", "Methods", "Rule"]
    sorts = [header.lower() for header in headers]
    if has_domain:
        headers.append("Host" if host_matching else "Subdomain")
        sorts.append("domain")

    with contextlib.suppress(ValueError):
        rows.sort(key=itemgetter(sorts.index(sort)))

    echo_table(
        rows,  # type: ignore
        tuple(headers),
        current_app.config.get("FLASK_CLI_EXTEND_TABLE_COLORS"),
    )


@cli.command("configs", short_help="Show app config parameters.")
@click.option("--default", "-d", is_flag=True, help="Show default config parameters.")
@click.option("--show-sensitive", "-S", is_flag=True, help="Show sensitive values.")
@click.option(
    "--diff",
    "-D",
    is_flag=True,
    help="Show differences between current app and default config settings.",
)
@click.option(
    "--namespace",
    "-n",
    type=click.STRING,
    multiple=True,
    help="Show config parameters that match the specified namespace/prefix.",
)
@click.option(
    "--no-namespace",
    "-N",
    type=click.STRING,
    multiple=True,
    help="Show config parameters excluding those that match the specified namespace/prefix.",
)
@with_appcontext
def configs_cmd(default: bool, diff: bool, show_sensitive: bool, namespace: tuple, no_namespace: tuple) -> None:
    """Display app config parameters.

    Args:
        default (bool): Show default config parameters.
        diff (bool): Show differences between current app and default config settings.
        show_sensitive (bool): Show sensitive values.
        namespace (tuple): Show config parameters that match the specified namespace/prefix.
        no_namespace (tuple): Show config parameters excluding those
        that match the specified namespace/prefix.
    """
    ctx = click.get_current_context()
    if default and diff:
        raise click.BadOptionUsage(
            "default diff",
            "Options --default and --diff are mutually exclusive.",
            ctx,
        )
    if namespace and no_namespace:
        raise click.BadOptionUsage(
            "namespace no_namespace",
            "Options --namespace and --no-namespace are mutually exclusive.",
            ctx,
        )

    conf: dict | Config = current_app.config.copy()

    if diff:
        conf = {
            k: v
            for k, v in current_app.config.items()
            if k not in current_app.default_config or v != current_app.default_config[k]
        }
    elif default:
        conf = current_app.default_config

    if namespace or no_namespace:
        conf = filter_dict(conf, namespace, no_namespace)

    if not show_sensitive:
        sensitive_keys = DEFAULT_SENSITIVE_CONFIG_KEYS + current_app.config.get(
            "FLASK_CLI_EXTEND_SENSITIVE_CONFIG_KEYS",
            [],
        )
        conf = mask_sensitive_values(conf, sensitive_keys)

    rows = [(k, str(v)) for k, v in conf.items()]
    rows.sort()
    headers = ("Key", "Value")
    echo_table(
        rows,  # type: ignore
        headers,
        current_app.config.get("FLASK_CLI_EXTEND_TABLE_COLORS"),
    )
