import click
from click.formatting import iter_rows, measure_table
from click.termui import _ansi_colors

DEFAULT_TABLE_COLORS = ["yellow", "cyan", "green", "blue", "magenta"]


def _prepare_table_colors(col_count: int, colors: list[str] | None = None) -> list[str]:
    """Prepare colors for the table.

    Args:
        col_count (int): Number of columns.
        colors (list[str] | None, optional): List of colors. Defaults to None.

    Returns:
        list[str]: List of colors for the table.
    """

    if isinstance(colors, list | tuple) and colors and set(colors).issubset(set(_ansi_colors)):
        prepared_colors = list(colors)
    else:
        prepared_colors = list(DEFAULT_TABLE_COLORS)

    if len(prepared_colors) < col_count:
        last_color = prepared_colors[-1] if prepared_colors else "white"
        prepared_colors.extend([last_color] * (col_count - len(prepared_colors)))

    return prepared_colors


def echo_table(
    rows: list[tuple[str, ...]],
    headers: tuple[str, ...],
    body_colors: list[str] | None = None,
    separator: str = "  ",
    bold_body: bool = False,
) -> None:
    """Echo a formatted table to the console.

    Args:
        rows (list[tuple[str, ...]]): Table rows.
        headers (tuple[str, ...]): Table headers.
        body_colors (list[str] | None, optional): Colors for the table body.
        separator (str, optional): Separator between columns. Defaults to "  ".
        bold_body (bool, optional): Whether to bold body. Defaults to False.
    """

    if not headers:
        secho("Table Headers are required.", category="error")
        raise click.Abort()

    if not rows:
        secho("No data to display.", category="warning")
        return

    col_count = len(headers)
    foreground_colors = _prepare_table_colors(col_count, body_colors)

    all_rows = [headers, *rows]
    widths = measure_table(all_rows)  # type: ignore
    rows = list(iter_rows(all_rows, col_count))  # type: ignore
    rows.insert(1, tuple("-" * width for width in widths))

    for i, row in enumerate(rows):
        click.echo(
            separator.join(
                (
                    click.style(cell.ljust(widths[j]), bold=True)
                    if i in [0, 1]
                    else click.style(cell.ljust(widths[j]), fg=foreground_colors[j], bold=bold_body)
                )
                for j, cell in enumerate(row)
            )
        )


def secho(message: str, category: str | None = None, **kwargs) -> None:
    """Info message echo template.

    Args:
        message (str): The message to print.
        category (str, optional): The category of the message.
        **kwargs: Additional keyword arguments to pass to `click.secho`.
    """
    match category:
        case "error":
            click.secho(f"ERROR: {message}", bold=True, fg="red", **kwargs)
        case "info":
            click.secho(f"INFO: {message}", bold=True, fg="green", **kwargs)
        case "warning":
            click.secho(f"WARNING: {message}", bold=True, fg="yellow", **kwargs)
        case _:
            click.secho(message, bold=True, **kwargs)
