from pathlib import Path
from secrets import randbelow
from typing import Any

from flask import current_app
from flask.sansio.blueprints import Blueprint

DEFAULT_SENSITIVE_CONFIG_KEYS = ["SECRET", "KEY", "PASSWORD", "TOKEN"]


def get_blueprint_rel_path(blueprint: Blueprint) -> Path:
	"""Returns the relative path of a Flask blueprint.

	Args:
	    blueprint (Blueprint): The blueprint to get the relative path for.

	Returns:
	    Path: The relative path of the blueprint.
	"""
	if Path(blueprint.root_path).is_relative_to(Path.cwd()):
		return Path(blueprint.root_path).relative_to(Path.cwd())
	return Path(blueprint.root_path).relative_to(current_app.root_path)


def filter_dict(items: dict[str, Any], namespace: tuple | None = None, no_namespace: tuple | None = None) -> dict:
	"""Filter a dictionary based on namespace and no_namespace criteria.

	Args:
	    items (dict[str, Any]): The dictionary to filter.
	    namespace (tuple | None, optional): Namespaces to include. Defaults to None.
	    no_namespace (tuple | None, optional): Namespaces to exclude. Defaults to None.

	Returns:
	    dict: The filtered dictionary.
	"""
	if not namespace and not no_namespace:
		return items

	namespaces = {ns.upper() for ns in namespace} if namespace else set()
	no_namespaces = {ns.upper() for ns in no_namespace} if no_namespace else set()

	filtered_items = {}
	for k, v in items.items():
		if namespaces and not any(k.startswith(ns) for ns in namespaces):
			continue
		if no_namespaces and any(k.startswith(ns) for ns in no_namespaces):
			continue
		filtered_items[k] = v

	return filtered_items


def mask_sensitive_values(items: dict[str, Any], sensitive_keys: list[str]) -> dict:
	"""Mask sensitive data in a dictionary.

	Args:
	    items (dict[str, Any]): The dictionary to mask.
	    sensitive_keys (list[str]): A list of sensitive keys.

	Returns:
	    dict: The masked dictionary.
	"""
	masked_items = items.copy()
	for k, v in masked_items.items():
		if any(s in k.upper() for s in sensitive_keys) and v is not None:
			masked_items[k] = f"{str(v)[:3]}{'*' * randbelow(29)}"
	return masked_items
